from __future__ import annotations

from pathlib import Path

from pyokru.models.user import (
    _login_by_email,
    _login_using_token,
    _refresh_session,
)
from pyokru.models.video import Video, _delete_video, _upload_video

_MOBILE_HEADERS = {
    "User-Agent": (
        "OKAndroid/22.3.21 b22032100 (Android 7.1.2; en_US; samsung SM-G977N"
        " Build/beyond1qlteue-user 7.1.2 LMY48Z 700220110 release-keys; mdpi"
        " 160dpi 540x960)"
    ),
    "Accept": "application/json",
    "Host": "api.ok.ru",
}


_WEB_HEADERS = {
    "Host": "ok.ru",
    "cache-control": "max-age=0",
    "sec-ch-ua": '"Not/A)Brand";v="99", "Opera";v="101", "Chromium";v="115"',
    "sec-ch-ua-mobile": "?0",
    "sec-ch-ua-platform": '"Windows"',
    "upgrade-insecure-requests": "1",
    "origin": "https://ok.ru",
    "content-type": "application/x-www-form-urlencoded",
    "user-agent": (
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML,"
        " like Gecko) Chrome/115.0.5790.171 Safari/537.36"
    ),
    "accept": (
        "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7"
    ),
    "sec-fetch-site": "same-origin",
    "sec-fetch-mode": "navigate",
    "sec-fetch-user": "?1",
    "sec-fetch-dest": "document",
    "referer": "https://ok.ru/dk?st.cmd=anonymMain",
}


class OkRu:
    def __init__(self: OkRu, token: str, email: str, password: str) -> None:
        self._token = token
        self._session = None
        self._email = email
        self._password = password
        self._headers = None
        self._session = None

    async def login(self: OkRu) -> None:
        self._headers = await _login_by_email(self._email, self._password, _WEB_HEADERS)

    async def refresh_web_token(self: OkRu) -> None:
        if not self._headers:
            raise Exception("Please login first")  # noqa: TRY002, TRY003
        self._headers = await _refresh_session(self._headers)

    async def refresh_session(self: OkRu) -> None:
        self._session = await _login_using_token(self._token, _MOBILE_HEADERS)

    async def upload_video(self: OkRu, file: Path, title: str) -> Video:
        if not self._session:
            raise Exception("Please refresh session first")  # noqa: TRY002, TRY003
        return await _upload_video(self._session, _MOBILE_HEADERS, title, file)

    async def delete_video(self: OkRu, video: Video | int) -> None:
        if not self._session:
            raise Exception("Please refresh session first")  # noqa: TRY002, TRY003
        if isinstance(video, Video):
            video = video.delete_id
        return await _delete_video(self._session, _MOBILE_HEADERS, video)

    async def fetch_video_info(
        self: OkRu, video: Video, only_one_type: bool = False
    ) -> bool:
        if not self._headers:
            raise Exception("Please login first")  # noqa: TRY003, TRY002
        return await video.fetch_info(self._headers, only_one_type)
