from __future__ import annotations

import re

from httpx import AsyncClient

from pyokru.utils import _str_cookies_to_dict

COOKIES = {
    "bci": "9017731843293820374",
    "_statid": "4ddff9cd-09dc-466a-a76b-8909bec83ca5",
    "__last_online": "1692364555426",
    "viewport": "728",
    "_ym_uid": "1692364553438676750",
    "_ym_d": "1692364553",
    "_ym_isad": "2",
}


async def _refresh_session(headers: dict[str, str]) -> dict[str, str]:
    data = 'a={"startupData":{"headBeacon":1}}'
    old_tkn = headers.pop("tkn")
    async with AsyncClient(
        base_url="https://ok.ru", headers=headers, http2=True
    ) as client:
        refresh_tkn_response = await client.post("/gwtlog", headers=headers, data=data)

        tkn_value = refresh_tkn_response.headers.get("tkn")
        gwt_cookies = {str(k): str(v) for k, v in refresh_tkn_response.cookies.items()}

        params = {
            "cmd": "PeriodicManager",
            "gwt.requested": "c25e364bT1692185206071",
            "p_sId": "0",
        }

        data = (
            "st.mpCheckTime=1692364826783:1692364826743:1692364826783:1692364826743"
            "&cpLCT=1692410254720&tlb.act=news&blocks=TD,PS,MP,VCP,NTF,FeedbackGrowl,MPC,MPC&p_NLP=1692410681687"
        )

        refresh_cookis_response = await client.post(
            "push", params=params, headers=headers, data=data
        )
        new_headers = {
            "tkn": (refresh_cookis_response.headers.get("tkn") or tkn_value or old_tkn)
        }
        cookies_dict = {
            str(k): str(v) for k, v in refresh_cookis_response.cookies.items()
        }

        old_cookies = _str_cookies_to_dict(headers["cookie"])

        cookies = old_cookies | cookies_dict | gwt_cookies
        cookies_string = "; ".join(
            [str(k) + "=" + str(v) for k, v in cookies.items()]
        ).strip()
        updated_headers = headers | new_headers
        updated_headers["cookie"] = cookies_string
        return updated_headers


async def _login_by_email(
    email: str, password: str, headers: dict[str, str]
) -> dict[str, str]:
    async with AsyncClient(
        base_url="https://ok.ru", headers=headers, http2=True
    ) as client:
        params = {"cmd": "AnonymLogin", "st.cmd": "anonymMain"}
        data = {
            "st.redirect": "",
            "st.asr": "",
            "st.posted": "set",
            "st.fJS": "on",
            "st.st.screenSize": "1366 x 768",
            "st.st.browserSize": "604",
            "st.st.flashVer": "0.0.0",
            "st.email": email,
            "st.password": password,
            "st.iscode": "false",
        }

        login_response = await client.post("/dk", data=data, params=params)
        headers["cookie"] = "; ".join(
            [str(k) + "=" + str(v) for k, v in login_response.cookies.items()]
        )
        token_response = await client.get(
            login_response.headers.get("location"), headers=headers
        )

        headers["tkn"] = _get_token(token_response.text)
        return headers


async def _login_using_token(token: str, headers: dict[str, str]) -> str:
    async with AsyncClient(
        base_url="https://api.ok.ru", headers=headers, http2=True
    ) as client:
        params = {
            "__screen": "feed_main",
            "application_key": "CBAFJIICABABABABA",
            "deviceId": (
                "INSTALL_ID=ac8b7d3b-83c1-42e7-9761-4ad0342f231b;DEVICE_ID=351564141331892;ANDROID_ID=ee8a0b402882410c;"
            ),
            "token": token,
            "verification_supported": "true",
            "verification_supported_v": "5",
        }
        response = await client.get("/api/auth/loginByToken", params=params)
        data: dict = response.json()
        if not data.get("error_code", False):
            return data["session_key"]
        raise ValueError("Invalid token")  # noqa: TRY003


def _get_token(html: str) -> str:
    pat = r"OK.tkn.set\(\'(.+)\'\);\<\/script\>"
    token = re.search(pat, html, flags=re.DOTALL)
    return token.group(1) if token else ""
